"""
This has the right number of neurons and synapses for the task, but only
presents the first of all three types of training data (3 items) to 
the system.  The net does categorise correctly.  
"""
import sys
import cPickle as pickle

import pyNN.nest as sim

RUN_DURATION = 800
TIME_STEP = 1.0
FIRING_WEIGHT = 0.1

neuronsInFirstLayer = 400
neuronsInSecondLayer = 3

#initialize the simulator. 
def init(): 
    sim.setup(timestep=TIME_STEP,min_delay=TIME_STEP,
                    max_delay=TIME_STEP, debug=0)

def createInputs():
    start = 10.0
    learnOffset = 1
    exampleOffset = 30
    epochOffset = 100
    numberTrainingEpochs = 5
    inputSpikeTimes0=inputSpikeTimes1=inputSpikeTimes2=inputSpikeTimes3 = []
    inputSpikeTimes4=inputSpikeTimes5=inputSpikeTimes6=inputSpikeTimes7 = []
    inputSpikeTimes8=[]

    for epoch in range (0,numberTrainingEpochs): 
        inputSpikeTimes0 = inputSpikeTimes0 + [start + (epoch*epochOffset)]
        inputSpikeTimes1 = inputSpikeTimes1 + [start + learnOffset + 
                                               (epoch*epochOffset)]
        inputSpikeTimes2 = inputSpikeTimes2 + [start + exampleOffset + 
                                               (epoch*epochOffset)]
        inputSpikeTimes3 = inputSpikeTimes3 + [start + exampleOffset+
                                               learnOffset+ (epoch*epochOffset)]

        inputSpikeTimes4 = inputSpikeTimes4 + [start + (2*exampleOffset) + 
                                               (epoch*epochOffset)]
        inputSpikeTimes5 = inputSpikeTimes5 + [start + (2*exampleOffset)+
                                               learnOffset+ (epoch*epochOffset)]

    inputSpikeTimes6 = [start + ((numberTrainingEpochs) * epochOffset)]
    inputSpikeTimes7 = [start + ((numberTrainingEpochs+1) * epochOffset)]
    inputSpikeTimes8 = [start + ((numberTrainingEpochs+2) * epochOffset)]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')

    spikeArray6 = {'spike_times': [inputSpikeTimes6]}
    spikeGen6=sim.Population(1, sim.SpikeSourceArray, spikeArray6,
                                   label='inputSpikes_6')
    spikeArray7 = {'spike_times': [inputSpikeTimes7]}
    spikeGen7=sim.Population(1, sim.SpikeSourceArray, spikeArray7,
                                   label='inputSpikes_7')
    spikeArray8 = {'spike_times': [inputSpikeTimes8]}
    spikeGen8=sim.Population(1, sim.SpikeSourceArray, spikeArray8,
                                   label='inputSpikes_8')


    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5, 
            spikeGen6,spikeGen7,spikeGen8]


def connectInput(spikeSource, cells, onCellList):
    connector = []
    for toNeuron in onCellList:
        connector = connector + [(0,toNeuron,FIRING_WEIGHT,TIME_STEP)]

    fromListConnector = sim.FromListConnector(connector)
    sim.Projection(spikeSource, cells, fromListConnector, 
                   receptor_type='excitatory')

def connectFirstToSecondLayer(fromCells, toCells):

    initialWeight = 0.001
    connector = []
    stdp_model = sim.STDPMechanism(
        timing_dependence=sim.SpikePairRule(tau_plus=16.7, 
                            tau_minus=3.7,A_plus=0.05, A_minus=0.002),
        weight_dependence=sim.AdditiveWeightDependence(w_min=0, w_max=0.2),
        weight=0.0,
        delay=1.0,
        dendritic_delay_fraction=float(1))

    sim.Projection(fromCells,toCells,
                   sim.AllToAllConnector(),
                   synapse_type = stdp_model)

def createDefaultNeurons(numberNeurons):
    cells=sim.Population(numberNeurons,sim.IF_cond_exp,cellparams={})
    return cells

def setupRecording(cells):
    cells.record({'spikes','v'})

def printResults(simCells,file):
    simCells.write_data(file+".pkl",'spikes')
    #simCells.write_data(file+"V.pkl",'v')
    #simCells.write_data(file+"W.pkl",'w')

##--main---
#args = sys.argv
#numberArgs = args.__len__()
#if (numberArgs == 2):
#    test = int(args[1])
#else:
#    test = 1


simName = "nest"
spinnVersion = -1

init()
spikeGenerators = createInputs()
firstInputGenerator = spikeGenerators[0]
firstOutputGenerator = spikeGenerators[1]
secondInputGenerator = spikeGenerators[2]
secondOutputGenerator = spikeGenerators[3]
thirdInputGenerator = spikeGenerators[4]
thirdOutputGenerator = spikeGenerators[5]
firstTestGenerator = spikeGenerators[6]
secondTestGenerator = spikeGenerators[7]
thirdTestGenerator = spikeGenerators[8]

firstLayerCells = createDefaultNeurons(neuronsInFirstLayer)
secondLayerCells = createDefaultNeurons(neuronsInSecondLayer)

#put in the data
input1 = [19,154,207,304]
input2 = [42,129,253,338]
input3 = [58,133,278,383]


#hook it up
connectInput(firstInputGenerator,firstLayerCells,input1)
connectInput(firstOutputGenerator,secondLayerCells,[0])
connectInput(secondInputGenerator,firstLayerCells,input2)
connectInput(secondOutputGenerator,secondLayerCells,[1])
connectInput(thirdInputGenerator,firstLayerCells,input3)
connectInput(thirdOutputGenerator,secondLayerCells,[2])

connectInput(firstTestGenerator,firstLayerCells,input1)
connectInput(secondTestGenerator,firstLayerCells,input2)
connectInput(thirdTestGenerator,firstLayerCells,input3)


connectFirstToSecondLayer(firstLayerCells,secondLayerCells)

setupRecording(firstLayerCells)
setupRecording(secondLayerCells)


sim.run(RUN_DURATION)

printResults(firstLayerCells,"output/first")
printResults(secondLayerCells,"output/second")
sim.reset()
