"""
This trains with 5 epochs of 01->X and 34->Y
The test works.
"""
import sys
import cPickle as pickle

import pyNN.nest as sim

RUN_DURATION = 700
TIME_STEP = 1.0
FIRING_WEIGHT = 0.1

neuronsInFirstLayer = 5
neuronsInSecondLayer = 2

#initialize the simulator. 
def init(): 
    sim.setup(timestep=TIME_STEP,min_delay=TIME_STEP,
                    max_delay=TIME_STEP, debug=0)

def createInputs():
    start = 10.0
    learnOffset = 1
    exampleOffset = 50
    epochOffset = 100
    numberTrainingEpochs = 5
    inputSpikeTimes0 = []
    inputSpikeTimes1 = []     
    inputSpikeTimes2 = []
    inputSpikeTimes3 = []
    for epoch in range (0,numberTrainingEpochs): 
        inputSpikeTimes0 = inputSpikeTimes0 + [start + (epoch*epochOffset)]
        inputSpikeTimes1 = inputSpikeTimes1 + [start + learnOffset + 
                                               (epoch*epochOffset)]
        inputSpikeTimes2 = inputSpikeTimes2 + [start + exampleOffset + 
                                               (epoch*epochOffset)]
        inputSpikeTimes3 = inputSpikeTimes3 + [start + exampleOffset+
                                               learnOffset+ (epoch*epochOffset)]

    inputSpikeTimes4 = [start + ((numberTrainingEpochs) * epochOffset)]
    inputSpikeTimes5 = [start + ((numberTrainingEpochs+1) * epochOffset)]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5]


def connectInput(spikeSource, cells, onCellList):
    connector = []
    for toNeuron in onCellList:
        connector = connector + [(0,toNeuron,FIRING_WEIGHT,TIME_STEP)]

    fromListConnector = sim.FromListConnector(connector)
    sim.Projection(spikeSource, cells, fromListConnector, 
                   receptor_type='excitatory')

def connectFirstToSecondLayer(fromCells, toCells):

    initialWeight = 0.001
    connector = []
    stdp_model = sim.STDPMechanism(
        timing_dependence=sim.SpikePairRule(tau_plus=16.7, 
                            tau_minus=3.7,A_plus=0.05, A_minus=0.002),
        weight_dependence=sim.AdditiveWeightDependence(w_min=0, w_max=0.2),
        weight=0.0,
        delay=1.0,
        dendritic_delay_fraction=float(1))

    sim.Projection(fromCells,toCells,
                   sim.AllToAllConnector(),
                   synapse_type = stdp_model)

def createDefaultNeurons(numberNeurons):
    cells=sim.Population(numberNeurons,sim.IF_cond_exp,cellparams={})
    return cells

def setupRecording(cells):
    cells.record({'spikes','v'})

def printResults(simCells,file):
    simCells.write_data(file+".pkl",'spikes')
    #simCells.write_data(file+"V.pkl",'v')
    #simCells.write_data(file+"W.pkl",'w')

##--main---
#args = sys.argv
#numberArgs = args.__len__()
#if (numberArgs == 2):
#    test = int(args[1])
#else:
#    test = 1


simName = "nest"
spinnVersion = -1

init()
spikeGenerators = createInputs()
firstInputGenerator = spikeGenerators[0]
firstOutputGenerator = spikeGenerators[1]
secondInputGenerator = spikeGenerators[2]
secondOutputGenerator = spikeGenerators[3]
thirdInputGenerator = spikeGenerators[4]
thirdOutputGenerator = spikeGenerators[5]
firstTestGenerator = spikeGenerators[6]
secondTestGenerator = spikeGenerators[7]
thirdTestGenerator = spikeGenerators[8]

firstLayerCells = createDefaultNeurons(neuronsInFirstLayer)
secondLayerCells = createDefaultNeurons(neuronsInSecondLayer)

#hook it up
connectInput(firstInputGenerator,firstLayerCells,[0,1])
connectInput(firstOutputGenerator,secondLayerCells,[0])
connectInput(secondInputGenerator,firstLayerCells,[3,4])
connectInput(secondOutputGenerator,secondLayerCells,[1])
connectInput(thirdInputGenerator,firstLayerCells,[2])
connectInput(thirdOutputGenerator,secondLayerCells,[2])


connectInput(firstTestGenerator,firstLayerCells,[0,1])
connectInput(secondTestGenerator,firstLayerCells,[3,4])
connectInput(thirdTestGenerator,firstLayerCells,[2])

connectFirstToSecondLayer(firstLayerCells,secondLayerCells)

setupRecording(firstLayerCells)
setupRecording(secondLayerCells)


sim.run(RUN_DURATION)

printResults(firstLayerCells,"output/first")
printResults(secondLayerCells,"output/second")
sim.reset()
