"""
This reads in the data from files.  It makes spike generators to
turn on the first layer, and associated generators to turn on the
second layer shortly thereafter, so it can learn by STDP.
The layers are connected by STDP synapses.
At the end of the run, it tests by using spike sources to turn on
the input layer, and the test is the output layer.
"""
import sys
import cPickle as pickle

import pyNN.nest as sim

#EXAMPLE_TIME = 30.0
EXAMPLE_TIME = 50.0
#NUMBER_TRAINING_EPOCHS = 5
NUMBER_TRAINING_EPOCHS = 6
#NUMBER_TRAINING_EPOCHS = 7
if ((EXAMPLE_TIME == 50.0) and (NUMBER_TRAINING_EPOCHS == 6)):
    END_OF_TRAINING = 105000
    RUN_DURATION =  140000
else:
    print "figure out the durations for", EXAMPLE_TIME, NUMBER_TRAINING_EPOCHS

TIME_STEP = 1.0
FIRING_WEIGHT = 0.1

neuronsInFirstLayer = 90
neuronsInSecondLayer = 2
inputFeatureBreadth = 0

firstSecond = -1 #global for printing out synaptic weights

def readData(fileName):
    data = []
    dataFileHandle = open(fileName,'r')
    inputLine = dataFileHandle.readline()
    while (not(inputLine == '')):
        dataItemStrings = inputLine.split()
        feature0 = int(dataItemStrings[0]) #ignore this one it's an ID
        feature1 = int(dataItemStrings[1])-1
        feature2 = int(dataItemStrings[2])-1
        feature3 = int(dataItemStrings[3])-1
        feature4 = int(dataItemStrings[4])-1
        feature5 = int(dataItemStrings[5])-1
        if (dataItemStrings[6] == '?'):
            feature6 = -1
        else:
            feature6 = int(dataItemStrings[6])-1
        feature7 = int(dataItemStrings[7])-1
        feature8 = int(dataItemStrings[8])-1
        feature9 = int(dataItemStrings[9])-1
        outputFeature = int(dataItemStrings[10])
        if (outputFeature == 2): 
            outputFeature = 0
        elif (outputFeature == 4): 
            outputFeature = 1
        else:
            print "wrong outputFeature ", outputFeature
        dataLine = [feature1,feature2,feature3,feature4,feature5,feature6,feature7,feature8,feature9,outputFeature]
        data = data + [dataLine]
        inputLine = dataFileHandle.readline()

    dataFileHandle.close()
    return data
    

#initialize the simulator. 
def init(): 
    sim.setup(timestep=TIME_STEP,min_delay=TIME_STEP,
                    max_delay=TIME_STEP, debug=0)

#Make spike sources to turn on the first layer, and the second layer for
#training.
def createTrainSources(data):
    numberDataItems = len(data)

    start = 10.0
    learnOffset = 1
    epochTime = EXAMPLE_TIME*numberDataItems
    spikeSources = []
    inputSpikeTimesMatrix = []
    outputSpikeTimesMatrix = []
    for dataOffset in range (0,numberDataItems):
        #print data[dataOffset]
        inputSpikeTimes = []
        outputSpikeTimes = []

        for epoch in range (0,NUMBER_TRAINING_EPOCHS): 
            inputSpikeTime = start+(dataOffset*EXAMPLE_TIME)+(epoch*epochTime)
            inputSpikeTimes = inputSpikeTimes + [inputSpikeTime]
            outputSpikeTime = inputSpikeTime + learnOffset
            outputSpikeTimes = outputSpikeTimes + [outputSpikeTime]
        inputSpikeTimesMatrix = inputSpikeTimesMatrix + [inputSpikeTimes]
        outputSpikeTimesMatrix = outputSpikeTimesMatrix + [outputSpikeTimes]


    inputSpikeGens = sim.Population(numberDataItems,sim.SpikeSourceArray,
                {'spike_times': inputSpikeTimesMatrix},label="inputSpikes")

    outputSpikeGens = sim.Population(numberDataItems,sim.SpikeSourceArray,
                {'spike_times': outputSpikeTimesMatrix},label="outputSpikes")

    #print outputSpikeTime

    return [inputSpikeGens,outputSpikeGens]

def createTestSources(data,startTime):
    numberDataItems = len(data)
    timeBetweenTests = EXAMPLE_TIME
    spikeSources = []
    testSpikeTimesMatrix = []
    for dataOffset in range (0,numberDataItems):
        testSpikeTimes = [[startTime + (dataOffset*timeBetweenTests)]]
        testSpikeTimesMatrix = testSpikeTimesMatrix + testSpikeTimes

    testSpikeGens = sim.Population(numberDataItems,sim.SpikeSourceArray,
                {'spike_times': testSpikeTimesMatrix},label="testSpikes")

    return testSpikeGens

def convertDataItemToSynapseDescription(dataLine,sourceNumber):
    inputConnector = []
    neuronsPerInputFeature = 10
    for dataFeature in range (0,9):
        dataFeatureValue = dataLine[dataFeature]
        if (dataFeatureValue >= 0):
            for neuronOffset in range((dataFeatureValue - inputFeatureBreadth),
                              (dataFeatureValue + inputFeatureBreadth+1)):
                if ((neuronOffset>=0)and(neuronOffset<neuronsPerInputFeature)):
                    toNeuron = neuronOffset+(dataFeature*neuronsPerInputFeature)
                    inputConnector = inputConnector + [(sourceNumber,toNeuron,
                        FIRING_WEIGHT,TIME_STEP)]

    return inputConnector

def connectInput(spikeSources, data,inputCells,outputCells):
    inputSpikeSources = spikeSources[0]
    outputSpikeSources = spikeSources[1]
    numberDataItems = len(data)
    for dataOffset in range (0,numberDataItems):
        dataItem = data[dataOffset]
        inputConnector=convertDataItemToSynapseDescription(dataItem,dataOffset)
        fromSource = dataOffset
        inputFromListConnector = sim.FromListConnector(inputConnector)
        sim.Projection(inputSpikeSources, inputCells, inputFromListConnector, 
                   receptor_type='excitatory')

        outputCategory = dataItem[9]
        outputConnector = [(fromSource,outputCategory,FIRING_WEIGHT,
                            TIME_STEP)]
            
        outputFromListConnector = sim.FromListConnector(outputConnector)
        sim.Projection(outputSpikeSources, outputCells,outputFromListConnector, 
                   receptor_type='excitatory')

def connectTest(testSpikeSources, data,inputCells):
    numberDataItems = len(data)
    for dataOffset in range (0,numberDataItems):
        dataItem = data[dataOffset]
        testConnector=convertDataItemToSynapseDescription(dataItem,dataOffset)
        testFromListConnector = sim.FromListConnector(testConnector)
        sim.Projection(testSpikeSources, inputCells, testFromListConnector, 
                   receptor_type='excitatory')

def connectFirstToSecondLayer(fromCells, toCells):
    global firstSecond
    connector = []
    stdp_model = sim.STDPMechanism(
        timing_dependence=sim.SpikePairRule(tau_plus=20.0,tau_minus=20.0,
                                            A_plus=0.006, A_minus=0.009),
        weight_dependence=sim.AdditiveWeightDependence(w_min=0.0, w_max=0.02),
        weight=0.0000,
        delay=1.0,
        dendritic_delay_fraction=float(1))

    firstSecond = sim.Projection(fromCells,toCells,
                   sim.AllToAllConnector(),
                   synapse_type = stdp_model)

def createDefaultNeurons(numberNeurons):
    cells=sim.Population(numberNeurons,sim.IF_cond_exp,cellparams={})
    return cells

def setupRecording(cells):
    cells.record({'spikes','v'})

def printResults(simCells,file):
    simCells.write_data(file+".pkl",'spikes')
    #simCells.write_data(file+"V.pkl",'v')
    #simCells.write_data(file+"W.pkl",'w')

##--main---
args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 3):
    trainFileName = args[1]
    testFileName = args[2]
else:
    trainFileName = "wiscTrain.txt"
    testFileName = "wiscTest.txt"

print trainFileName, testFileName


simName = "nest"
spinnVersion = -1

trainData = readData(trainFileName)
testData = readData(testFileName)


init()
trainSpikeGenerators = createTrainSources(trainData)
testSpikeGenerators = createTestSources(testData,END_OF_TRAINING)

firstLayerCells = createDefaultNeurons(neuronsInFirstLayer)
secondLayerCells = createDefaultNeurons(neuronsInSecondLayer)

#hook it up
connectInput(trainSpikeGenerators,trainData,firstLayerCells,secondLayerCells)
connectTest(testSpikeGenerators,testData,firstLayerCells)
connectFirstToSecondLayer(firstLayerCells,secondLayerCells)

setupRecording(firstLayerCells)
setupRecording(secondLayerCells)

sim.run(RUN_DURATION)

printResults(firstLayerCells,"output/firstWisc")
printResults(secondLayerCells,"output/secondWisc")

#firstSecondSynapseWeights = firstSecond.get(["weight"],format="list")
#print firstSecondSynapseWeights

sim.reset()
